@@ -266,7 +266,7 @@ static pgqsEntry *pgqs_process_opexpr(OpExpr *expr, pgqsWalkerContext *context);
 static pgqsEntry *pgqs_process_scalararrayopexpr(ScalarArrayOpExpr *expr, pgqsWalkerContext *context);
 static pgqsEntry *pgqs_process_booltest(BooleanTest *expr, pgqsWalkerContext *context);
 static void pgqs_collectNodeStats(PlanState *planstate, List *ancestors, pgqsWalkerContext *context);
-static void pgqs_collectMemberNodeStats(List *plans, PlanState **planstates, List *ancestors, pgqsWalkerContext *context);
+static void pgqs_collectMemberNodeStats(int nplans, PlanState **planstates, List *ancestors, pgqsWalkerContext *context);
 static void pgqs_collectSubPlanStats(List *plans, List *ancestors, pgqsWalkerContext *context);
 static uint32 hashExpr(Expr *expr, pgqsWalkerContext *context, bool include_const);
 static void exprRepr(Expr *expr, StringInfo buffer, pgqsWalkerContext *context, bool include_const);
@@ -728,7 +728,7 @@ pgqs_ExecutorEnd(QueryDesc *queryDesc)
 			 * that much entries in localhash in the first place.
 			 */
 			int			nvictims = hash_get_num_entries(pgqs_localhash) -
-			PGQS_MAX_LOCAL_ENTRIES;
+				PGQS_MAX_LOCAL_ENTRIES;
 
 			if (nvictims > 0)
 				pgqs_localentry_dealloc(nvictims);
@@ -857,13 +857,16 @@ pgqs_queryentry_dealloc(void)
 }
 
 /*
- * Remove the requested number of entries from pgqs_localhash */
+ * Remove the requested number of entries from pgqs_localhash.  Since the
+ * entries are all coming from the same query, remove them without any specific
+ * sort.
+ */
 static void
 pgqs_localentry_dealloc(int nvictims)
 {
 	pgqsEntry  *localentry;
 	HASH_SEQ_STATUS local_hash_seq;
-	pgqsHashKey *victims[nvictims];
+	pgqsHashKey **victims;
 	bool		need_seq_term = true;
 	int			i,
 				ptr = 0;
@@ -871,6 +874,8 @@ pgqs_localentry_dealloc(int nvictims)
 	if (nvictims <= 0)
 		return;
 
+	victims = palloc(sizeof(pgqsHashKey *) * nvictims);
+
 	hash_seq_init(&local_hash_seq, pgqs_localhash);
 	while (nvictims-- >= 0)
 	{
@@ -891,6 +896,8 @@ pgqs_localentry_dealloc(int nvictims)
 
 	for (i = 0; i < ptr; i++)
 		hash_search(pgqs_localhash, victims[i], HASH_REMOVE, NULL);
+
+	pfree(victims);
 }
 
 static void
@@ -993,27 +1000,27 @@ pgqs_collectNodeStats(PlanState *planstate, List *ancestors, pgqsWalkerContext *
 	switch (nodeTag(plan))
 	{
 		case T_ModifyTable:
-			pgqs_collectMemberNodeStats(((ModifyTable *) plan)->plans,
+			pgqs_collectMemberNodeStats(((ModifyTableState *) planstate)->mt_nplans,
 										((ModifyTableState *) planstate)->mt_plans,
 										ancestors, context);
 			break;
 		case T_Append:
-			pgqs_collectMemberNodeStats(((Append *) plan)->appendplans,
+			pgqs_collectMemberNodeStats(((AppendState *) planstate)->as_nplans,
 										((AppendState *) planstate)->appendplans,
 										ancestors, context);
 			break;
 		case T_MergeAppend:
-			pgqs_collectMemberNodeStats(((MergeAppend *) plan)->mergeplans,
+			pgqs_collectMemberNodeStats(((MergeAppendState *) planstate)->ms_nplans,
 										((MergeAppendState *) planstate)->mergeplans,
 										ancestors, context);
 			break;
 		case T_BitmapAnd:
-			pgqs_collectMemberNodeStats(((BitmapAnd *) plan)->bitmapplans,
+			pgqs_collectMemberNodeStats(((BitmapAndState *) planstate)->nplans,
 										((BitmapAndState *) planstate)->bitmapplans,
 										ancestors, context);
 			break;
 		case T_BitmapOr:
-			pgqs_collectMemberNodeStats(((BitmapOr *) plan)->bitmapplans,
+			pgqs_collectMemberNodeStats(((BitmapOrState *) planstate)->nplans,
 										((BitmapOrState *) planstate)->bitmapplans,
 										ancestors, context);
 			break;
@@ -1030,10 +1037,9 @@ pgqs_collectNodeStats(PlanState *planstate, List *ancestors, pgqsWalkerContext *
 }
 
 static void
-pgqs_collectMemberNodeStats(List *plans, PlanState **planstates,
+pgqs_collectMemberNodeStats(int nplans, PlanState **planstates,
 							List *ancestors, pgqsWalkerContext *context)
 {
-	int			nplans = list_length(plans);
 	int			j;
 
 	for (j = 0; j < nplans; j++)
@@ -1955,6 +1961,12 @@ pgqs_set_planstates(PlanState *planstate, pgqsWalkerContext *context)
 	/* index_tlist is set only if it's an IndexOnlyScan */
 	if (IsA(planstate->plan, IndexOnlyScan))
 		context->index_tlist = ((IndexOnlyScan *) planstate->plan)->indextlist;
+#if PG_VERSION_NUM >= 90500
+	else if (IsA(planstate->plan, ForeignScan))
+		context->index_tlist = ((ForeignScan *) planstate->plan)->fdw_scan_tlist;
+	else if (IsA(planstate->plan, CustomScan))
+		context->index_tlist = ((CustomScan *) planstate->plan)->custom_scan_tlist;
+#endif
 	else
 		context->index_tlist = NIL;
 }
@@ -1998,7 +2010,7 @@ pgqs_resolve_var(Var *var, pgqsWalkerContext *context)
 		}
 	}
 
-	Assert(!(IsA(var, Var) &&IS_SPECIAL_VARNO(var->varno)));
+	Assert(!(IsA(var, Var) && IS_SPECIAL_VARNO(var->varno)));
 
 	/* If the result is something OTHER than a var, replace it by a constexpr */
 	if (!IsA(var, Var))
